local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local inventory = include("sim/inventory")
local abilityutil = include( "sim/abilities/abilityutil" )

local melee_tooltip = class()

function melee_tooltip:init( hud, abilityOwner, abilityUser, targetUnitID )
	self._hud = hud
	self._abilityOwner = abilityOwner
	self._abilityUser = abilityUser
	self._targetUnitID = targetUnitID
end

function melee_tooltip:setPosition( wx, wy )
	self._panel:setPosition( self._hud._screen:wndToUI( wx, wy ))
end

function melee_tooltip:getScreen()
	return self._hud._screen
end

function melee_tooltip:activate( screen )
	local combat_panel = include( "hud/combat_panel" )
	local sim = self._hud._game.simCore

	self._panel = combat_panel( self._hud, self._hud._screen )
	self._panel:refreshMelee( self._abilityOwner, self._abilityUser, sim:getUnit( self._targetUnitID ), self._abilityOwner)
end

function melee_tooltip:deactivate()
	self._hud._game.boardRig:getUnitRig( self._targetUnitID )._prop:setRenderFilter( nil )
	self._panel:setVisible( false )
end

local W93_guard_kick =
{
	name = STRINGS.ITEMSEXTEND.UI.GUARD_KICK,
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_sneak_small.png",
	usesAction = true,
	proxy = true,

	onTooltip = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		return melee_tooltip( hud, abilityOwner, abilityUser, targetUnitID )
	end,
		
	getName = function( self, sim, unit )
		return STRINGS.ITEMSEXTEND.UI.GUARD_KICK
	end,

	getProfileIcon = function( self, sim, abilityOwner )
		return self.profile_icon
	end,

	acquireTargets = function( self, targets, game, sim, unit )
		local userUnit = unit:getUnitOwner()
		if not userUnit then
			return targets.unitTarget( game, {}, self, unit, nil )
		end
		assert( userUnit, tostring(unit and unit:getName())..", "..tostring(unit:getLocation()) )
		local cell = sim:getCell( userUnit:getLocation() )
        	local cells = { cell }
		for dir, exit in pairs(cell.exits) do
			if simquery.isOpenExit( exit ) then
				table.insert( cells, exit.cell )
			end
		end

		local units = {}
		for i, cell in pairs(cells) do
			for _, cellUnit in ipairs( cell.units ) do
				if unit:getTraits().installed and cellUnit:getTraits().isGuard and not cellUnit:getTraits().isDrone and not cellUnit:isDown() then
					table.insert( units, cellUnit )
				end
			end
		end

		return targets.unitTarget( game, units, self, unit, userUnit )
	end,
		
	canUseAbility = function( self, sim, unit, userUnit, targetID )
		if not userUnit then
			return false
		end

		if userUnit:getAP() < 1 then 
			return false, STRINGS.UI.COMBAT_PANEL_FAIL_KO, STRINGS.UI.COMBAT_PANEL_NO_ATTACK
		end

		if userUnit:getTraits().mpAttackRestrict and userUnit:getTraits().mpAttackRestrict > userUnit:getMP() then
			return false, string.format(STRINGS.FUNCLIB.UI.COMBAT_PANEL_MP, userUnit:getTraits().mpAttackRestrict)
		end

		if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 then
			return false, util.sformat(STRINGS.UI.COMBAT_PANEL_COOLDOWN, unit:getTraits().cooldown), STRINGS.UI.COMBAT_PANEL_COOLDOWN_2
		end
		
		if unit:getTraits().usesCharges and unit:getTraits().charges < 1 then
			return false, STRINGS.UI.COMBAT_PANEL_NEED_CHARGES, STRINGS.UI.COMBAT_PANEL_NEED_CHARGES_2			
		end

		if unit:getTraits().pwrCost and  userUnit:getPlayerOwner():getCpus() < unit:getTraits().pwrCost then
			return false, STRINGS.UI.COMBAT_PANEL_FAIL_KO, STRINGS.UI.COMBAT_PANEL_NO_PWR, STRINGS.UI.FLY_TXT.NOT_ENOUGH_PWR
		end

		if targetID then
			local targetUnit = sim:getUnit( targetID )
			if not targetUnit then
				return false
			end

			if not sim:getParams().difficultyOptions.meleeFromFront and not targetUnit:getTraits().modifyingExit and not targetUnit:getTraits().turning and not targetUnit:getTraits().movePath and sim:canUnitSeeUnit( targetUnit, userUnit ) then
				return false, STRINGS.UI.COMBAT_PANEL_FAIL_KO, STRINGS.UI.COMBAT_PANEL_SEEN
			end

			local dmg = simquery.calculateMeleeDamage(sim, unit, targetUnit)
			if dmg <= 0 and targetUnit:getArmor() > 0 then
				return false, STRINGS.UI.COMBAT_PANEL_ARMORED
			end
		end

        	return abilityutil.checkRequirements( unit, userUnit )
	end,
		
	executeAbility = function( self, sim, unit, userUnit, target )
		local userUnit = unit:getUnitOwner()
		local target = sim:getUnit(target)			
			
		local x0,y0 = userUnit:getLocation()
		local x1,y1 = target:getLocation()
  		local newFacing = simquery.getDirectionFromDelta(x1-x0,y1-y0) 

		userUnit:resetAllAiming()
		simquery.suggestAgentFacing(userUnit, newFacing)

		if not userUnit:getTraits().noDoorAnim then 
			sim:dispatchEvent( simdefs.EV_UNIT_USEDOOR, { unitID = userUnit:getID(), facing = newFacing, exitOp=simdefs.EXITOP_BREAK_DOOR } )	
		end

		local dmgt =
		{
			damage = simquery.calculateMeleeDamage(sim, unit, target),
			ko = true,
			noTargetAlert = false,
		}
		sim:hitUnit( userUnit, target, dmgt )

		if not userUnit:getTraits().noDoorAnim then
			sim:dispatchEvent( simdefs.EV_UNIT_USEDOOR_PST, { unitID = userUnit:getID(), facing = newFacing, exitOp=simdefs.EXITOP_BREAK_DOOR } )
		end

		if userUnit:getTraits().convertKOtoPWR then
			userUnit:getPlayerOwner():addCPUs(dmgt.damage, sim, x0, y0, 30 )						
			sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat(userUnit:getTraits().convertKOtoPWR, dmgt.damage), x=x0,y=y0, color=cdefs.AUGMENT_TXT_COLOR} )
		end

		userUnit:useAP( sim )
		inventory.useItem( sim, userUnit, unit )
		if userUnit:isValid() then
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit  } )
		end
		simquery.LEVER_resetStatus( sim, userUnit, "ANY", false )
	end,
}

return W93_guard_kick