local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local inventory = include("sim/inventory")

-------------------------------------------------------------
--
local function getTargetUnits( sim, userUnit, x0, y0, range )
    local units = {}
	local cells = simquery.fillCircle( sim, x0, y0, range, 0)

	for i, cell in ipairs(cells) do
		for i, cellUnit in ipairs( cell.units ) do
			if not simquery.isEnemyAgent(userUnit:getPlayerOwner(), cellUnit) and simquery.isAgent( cellUnit ) and cellUnit ~= userUnit then
                		table.insert( units, cellUnit )
			end
		end
	end
    	return units
end

-------------------------------------------------------------
-- Special cloak tooltip for showing range

local cloak_tooltip = class( abilityutil.hotkey_tooltip )

function cloak_tooltip:init( hud, unit, range, ... )
	abilityutil.hotkey_tooltip.init( self, ... )
	self._game = hud._game
	self._unit = unit
	self._range = range
    	if range then
        	local x0, y0 = unit:getLocation() 
        	self._targets = getTargetUnits( hud._game.simCore, unit, x0, y0, range)
    	end
end

function cloak_tooltip:activate( screen )
	abilityutil.hotkey_tooltip.activate( self, screen )
	if self._range then
		local x0, y0 = self._unit:getLocation()
		local coords = simquery.rasterCircle( self._game.simCore, x0, y0, self._range )
		self._hiliteID = self._game.boardRig:hiliteCells( coords, cdefs.HILITE_TARGET_COLOR )
        	for i, target in ipairs( self._targets ) do
            		self._game.boardRig:getUnitRig( target:getID() ):getProp():setRenderFilter( cdefs.RENDER_FILTERS["focus_target"] )
        	end
	end
end

function cloak_tooltip:deactivate()
	abilityutil.hotkey_tooltip.deactivate( self )
	if self._range then
		self._game.boardRig:unhiliteCells( self._hiliteID )	
		self._hiliteID = nil
        	for i, target in ipairs( self._targets ) do
            		self._game.boardRig:getUnitRig( target:getID() ):refreshRenderFilter()
        	end
	end
end


local W93_useInvisiCloak =
{
	name = STRINGS.ABILITIES.CLOAK,
	onTooltip = function( self, hud, sim, abilityOwner, abilityUser )
		local i, t = abilityOwner:countAugments("W93_augment_subdermal_cloak")
		local unit = t[1]
		return cloak_tooltip( hud, abilityUser, unit:getTraits().range, self, sim, abilityOwner, STRINGS.ITEMSEXTEND.UI.SUBDERMAL_CLOAK_PROJECTOR_DESC )
	end,

	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_holomesh_Prism.png",
	alwaysShow = true,
	HUDpriority = 6,

	getName = function( self, sim, unit )
		return STRINGS.ABILITIES.CLOAK_USE
	end,

	onSpawnAbility = function( self, sim, unit )
        	sim:addTrigger( simdefs.TRG_START_TURN, self )
	end,
        
	onDespawnAbility = function( self, sim, unit )
        	sim:removeTrigger( simdefs.TRG_START_TURN, self )
	end,

	onTrigger = function ( self, sim, evType, evData )
		if self.abilityOwner and not self.abilityOwner:getTraits().invisible then
			self.abilityOwner:getTraits().cloakOn = nil
		end
		if evType == simdefs.TRG_START_TURN and sim:getPC() == sim:getCurrentPlayer() and self.abilityOwner and self.abilityOwner:getTraits().cloakOn then
			local i, t = self.abilityOwner:countAugments("W93_augment_subdermal_cloak")
			local unit = t[1]
			if sim:getPC():getCpus() < unit:getTraits().CPUperTurn then
				self.abilityOwner:getTraits().cloakOn = nil
				self.abilityOwner:setInvisible(false)
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = self.abilityOwner } )
				sim:processReactions(self.abilityOwner)
			else
				sim:getPC():addCPUs( -unit:getTraits().CPUperTurn )
				sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=string.format(STRINGS.ITEMSEXTEND.UI.SUBDERMAL_CLOAK_PROJECTOR_WARNING, unit:getTraits().CPUperTurn), color=cdefs.COLOR_PLAYER_WARNING, sound = "SpySociety/Actions/mainframe_PWRreverse_off", icon=self.icon } )
			end
		end
	end,

	canUseAbility = function( self, sim, userUnit )
		local i, t = userUnit:countAugments("W93_augment_subdermal_cloak")
		local unit = t[1]

		if i == 1 then
			if unit:getTraits().cooldown and unit:getTraits().cooldown > 0 and not userUnit:getTraits().cloakOn  then
				return false, util.sformat(STRINGS.UI.REASON.COOLDOWN, unit:getTraits().cooldown)
			end

			if unit:getTraits().usesCharges and unit:getTraits().charges < 1 and not userUnit:getTraits().cloakOn  then
				return false, util.sformat(STRINGS.UI.REASON.CHARGES)
			end

			if unit:getTraits().pwrCost and unit:getPlayerOwner():getCpus() < unit:getTraits().pwrCost and not userUnit:getTraits().cloakOn then
				return false,  STRINGS.UI.REASON.NOT_ENOUGH_PWR
			end				
	
			if not unit:getTraits().cloakInVision and not userUnit:getTraits().cloakOn then 
				if sim:canPlayerSeeUnit( sim:getNPC(), userUnit ) then 
					return false, STRINGS.UI.REASON.CANT_CLOAK_IN_VISION
				end
			end

			return abilityutil.checkRequirements( unit, userUnit )
		else
			return false
		end		
	end,

	executeAbility = function( self, sim, userUnit )
		local i, t = userUnit:countAugments("W93_augment_subdermal_cloak")
		local unit = t[1]

		if not self.abilityOwner then
			self.abilityOwner = userUnit
		end

		if userUnit:getTraits().cloakOn then
			userUnit:setInvisible(false)
			userUnit:getTraits().cloakOn = nil
		else
			userUnit:getTraits().cloakOn = true
			userUnit:setInvisible(true, unit:getTraits().duration)
			userUnit:resetAllAiming()

			if unit:getTraits().cloakDistanceMax then
				userUnit:getTraits().cloakDistance = unit:getTraits().cloakDistanceMax
			else
				userUnit:getTraits().cloakDistance = nil
			end

			if unit:getTraits().range then
                		local x0, y0 = userUnit:getLocation()
                		local units = getTargetUnits( sim, userUnit, x0, y0, unit:getTraits().range )
                		for i, cellUnit in ipairs( units ) do
                    			cellUnit:setInvisible( true, unit:getTraits().duration )
                		end
			end

			inventory.useItem( sim, userUnit, unit )
		end
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit  } )
		sim:processReactions(userUnit)
		simquery.LEVER_resetStatus( sim, userUnit, "LEGIT", true )
	end,
}
return W93_useInvisiCloak